@@ -77,6 +77,7 @@ typedef struct _xmpp_connlist_t {
 struct _xmpp_ctx_t {
     const xmpp_mem_t *mem;
     const xmpp_log_t *log;
+    int verbosity;
 
     xmpp_rand_t *rand;
     xmpp_loop_status_t loop_status;
@@ -87,33 +88,24 @@ struct _xmpp_ctx_t {
 };
 
 /* convenience functions for accessing the context */
-void *xmpp_alloc(const xmpp_ctx_t *const ctx, const size_t size);
-void *xmpp_realloc(const xmpp_ctx_t *const ctx, void *p, const size_t size);
-char *xmpp_strdup(const xmpp_ctx_t *const ctx, const char *const s);
+void *xmpp_alloc(const xmpp_ctx_t *ctx, size_t size);
+void *xmpp_realloc(const xmpp_ctx_t *ctx, void *p, size_t size);
+char *xmpp_strdup(const xmpp_ctx_t *ctx, const char *s);
+char *xmpp_strndup(const xmpp_ctx_t *ctx, const char *s, size_t len);
 
-void xmpp_log(const xmpp_ctx_t *const ctx,
+void xmpp_log(const xmpp_ctx_t *ctx,
               const xmpp_log_level_t level,
-              const char *const area,
-              const char *const fmt,
+              const char *area,
+              const char *fmt,
               va_list ap);
 
 /* wrappers for xmpp_log at specific levels */
-void xmpp_error(const xmpp_ctx_t *const ctx,
-                const char *const area,
-                const char *const fmt,
-                ...);
-void xmpp_warn(const xmpp_ctx_t *const ctx,
-               const char *const area,
-               const char *const fmt,
-               ...);
-void xmpp_info(const xmpp_ctx_t *const ctx,
-               const char *const area,
-               const char *const fmt,
-               ...);
-void xmpp_debug(const xmpp_ctx_t *const ctx,
-                const char *const area,
-                const char *const fmt,
-                ...);
+void xmpp_error(const xmpp_ctx_t *ctx, const char *area, const char *fmt, ...);
+void xmpp_warn(const xmpp_ctx_t *ctx, const char *area, const char *fmt, ...);
+void xmpp_info(const xmpp_ctx_t *ctx, const char *area, const char *fmt, ...);
+void xmpp_debug(const xmpp_ctx_t *ctx, const char *area, const char *fmt, ...);
+void xmpp_debug_verbose(
+    int level, const xmpp_ctx_t *ctx, const char *area, const char *fmt, ...);
 
 /** connection **/
 
@@ -143,6 +135,7 @@ struct _xmpp_send_queue_t {
 #define SASL_MASK_SCRAMSHA1 (1 << 3)
 #define SASL_MASK_SCRAMSHA256 (1 << 4)
 #define SASL_MASK_SCRAMSHA512 (1 << 5)
+#define SASL_MASK_EXTERNAL (1 << 6)
 
 #define SASL_MASK_SCRAM \
     (SASL_MASK_SCRAMSHA1 | SASL_MASK_SCRAMSHA256 | SASL_MASK_SCRAMSHA512)
@@ -153,7 +146,7 @@ enum {
     XMPP_PORT_COMPONENT = 5347,
 };
 
-typedef void (*xmpp_open_handler)(xmpp_conn_t *const conn);
+typedef void (*xmpp_open_handler)(xmpp_conn_t *conn);
 
 struct _xmpp_conn_t {
     unsigned int ref;
@@ -176,11 +169,16 @@ struct _xmpp_conn_t {
     int tls_mandatory;
     int tls_legacy_ssl;
     int tls_trust;
+    char *tls_cafile;
+    char *tls_capath;
+    char *tls_client_cert;
+    char *tls_client_key;
     int tls_failed;   /* set when tls fails, so we don't try again */
     int sasl_support; /* if true, field is a bitfield of supported
                          mechanisms */
     int auth_legacy_enabled;
     int secured; /* set when stream is secured with TLS */
+    xmpp_certfail_handler certfail_handler;
 
     /* if server returns <bind/> or <session/> we must do them */
     int bind_required;
@@ -225,13 +223,13 @@ struct _xmpp_conn_t {
     xmpp_handlist_t *handlers;
 };
 
-void conn_disconnect(xmpp_conn_t *const conn);
-void conn_disconnect_clean(xmpp_conn_t *const conn);
-void conn_established(xmpp_conn_t *const conn);
-void conn_open_stream(xmpp_conn_t *const conn);
-int conn_tls_start(xmpp_conn_t *const conn);
-void conn_prepare_reset(xmpp_conn_t *const conn, xmpp_open_handler handler);
-void conn_parser_reset(xmpp_conn_t *const conn);
+void conn_disconnect(xmpp_conn_t *conn);
+void conn_disconnect_clean(xmpp_conn_t *conn);
+void conn_established(xmpp_conn_t *conn);
+void conn_open_stream(xmpp_conn_t *conn);
+int conn_tls_start(xmpp_conn_t *conn);
+void conn_prepare_reset(xmpp_conn_t *conn, xmpp_open_handler handler);
+void conn_parser_reset(xmpp_conn_t *conn);
 
 typedef enum {
     XMPP_STANZA_UNKNOWN,
@@ -256,32 +254,32 @@ struct _xmpp_stanza_t {
 };
 
 /* handler management */
-void handler_fire_stanza(xmpp_conn_t *const conn, xmpp_stanza_t *const stanza);
-uint64_t handler_fire_timed(xmpp_ctx_t *const ctx);
+void handler_fire_stanza(xmpp_conn_t *conn, xmpp_stanza_t *stanza);
+uint64_t handler_fire_timed(xmpp_ctx_t *ctx);
 void handler_reset_timed(xmpp_conn_t *conn, int user_only);
-void handler_add_timed(xmpp_conn_t *const conn,
+void handler_add_timed(xmpp_conn_t *conn,
                        xmpp_timed_handler handler,
-                       const unsigned long period,
-                       void *const userdata);
-void handler_add_id(xmpp_conn_t *const conn,
+                       unsigned long period,
+                       void *userdata);
+void handler_add_id(xmpp_conn_t *conn,
                     xmpp_handler handler,
-                    const char *const id,
-                    void *const userdata);
-void handler_add(xmpp_conn_t *const conn,
+                    const char *id,
+                    void *userdata);
+void handler_add(xmpp_conn_t *conn,
                  xmpp_handler handler,
-                 const char *const ns,
-                 const char *const name,
-                 const char *const type,
-                 void *const userdata);
+                 const char *ns,
+                 const char *name,
+                 const char *type,
+                 void *userdata);
 void handler_system_delete_all(xmpp_conn_t *conn);
 
 /* utility functions */
-void disconnect_mem_error(xmpp_conn_t *const conn);
+void disconnect_mem_error(xmpp_conn_t *conn);
 
 /* auth functions */
-void auth_handle_open(xmpp_conn_t *const conn);
-void auth_handle_component_open(xmpp_conn_t *const conn);
-void auth_handle_open_raw(xmpp_conn_t *const conn);
-void auth_handle_open_stub(xmpp_conn_t *const conn);
+void auth_handle_open(xmpp_conn_t *conn);
+void auth_handle_component_open(xmpp_conn_t *conn);
+void auth_handle_open_raw(xmpp_conn_t *conn);
+void auth_handle_open_stub(xmpp_conn_t *conn);
 
 #endif /* __LIBSTROPHE_COMMON_H__ */
