@@ -48,10 +48,13 @@
 #define CONNECT_TIMEOUT 5000 /* 5 seconds */
 #endif
 
-static int _disconnect_cleanup(xmpp_conn_t *const conn, void *const userdata);
-static char *_conn_build_stream_tag(xmpp_conn_t *const conn,
+static int _disconnect_cleanup(xmpp_conn_t *conn, void *userdata);
+static char *_conn_build_stream_tag(xmpp_conn_t *conn,
                                     char **attributes,
                                     size_t attributes_len);
+static int _conn_open_stream_with_attributes(xmpp_conn_t *conn,
+                                             char **attributes,
+                                             size_t attributes_len);
 static void _conn_attributes_new(xmpp_conn_t *conn,
                                  char **attrs,
                                  char ***attributes,
@@ -59,24 +62,22 @@ static void _conn_attributes_new(xmpp_conn_t *conn,
 static void _conn_attributes_destroy(xmpp_conn_t *conn,
                                      char **attributes,
                                      size_t attributes_len);
-static void
-_handle_stream_start(char *name, char **attrs, void *const userdata);
-static void _handle_stream_end(char *name, void *const userdata);
-static void _handle_stream_stanza(xmpp_stanza_t *stanza, void *const userdata);
-static unsigned short _conn_default_port(xmpp_conn_t *const conn,
+static void _handle_stream_start(char *name, char **attrs, void *userdata);
+static void _handle_stream_end(char *name, void *userdata);
+static void _handle_stream_stanza(xmpp_stanza_t *stanza, void *userdata);
+static unsigned short _conn_default_port(xmpp_conn_t *conn,
                                          xmpp_conn_type_t type);
-static void _conn_reset(xmpp_conn_t *const conn);
-static int _conn_connect(xmpp_conn_t *const conn,
-                         const char *const domain,
-                         const char *const host,
+static void _conn_reset(xmpp_conn_t *conn);
+static int _conn_connect(xmpp_conn_t *conn,
+                         const char *domain,
+                         const char *host,
                          unsigned short port,
                          xmpp_conn_type_t type,
                          xmpp_conn_handler callback,
-                         void *const userdata);
+                         void *userdata);
+static int _send_raw(xmpp_conn_t *conn, char *data, size_t len);
 
-void xmpp_send_error(xmpp_conn_t *const conn,
-                     xmpp_error_type_t const type,
-                     char *const text)
+void xmpp_send_error(xmpp_conn_t *conn, xmpp_error_type_t type, char *text)
 {
     xmpp_stanza_t *error = xmpp_error_new(conn->ctx, type, text);
 
@@ -93,7 +94,7 @@ void xmpp_send_error(xmpp_conn_t *const conn,
  *
  *  @ingroup Connections
  */
-xmpp_conn_t *xmpp_conn_new(xmpp_ctx_t *const ctx)
+xmpp_conn_t *xmpp_conn_new(xmpp_ctx_t *ctx)
 {
     xmpp_conn_t *conn = NULL;
     xmpp_connlist_t *tail, *item;
@@ -143,9 +144,14 @@ xmpp_conn_t *xmpp_conn_new(xmpp_ctx_t *const ctx)
         conn->tls_legacy_ssl = 0;
         conn->tls_trust = 0;
         conn->tls_failed = 0;
+        conn->tls_cafile = NULL;
+        conn->tls_capath = NULL;
+        conn->tls_client_cert = NULL;
+        conn->tls_client_key = NULL;
         conn->sasl_support = 0;
         conn->auth_legacy_enabled = 0;
         conn->secured = 0;
+        conn->certfail_handler = NULL;
 
         conn->bind_required = 0;
         conn->session_required = 0;
@@ -201,7 +207,7 @@ xmpp_conn_t *xmpp_conn_new(xmpp_ctx_t *const ctx)
  *
  *  @ingroup Connections
  */
-xmpp_conn_t *xmpp_conn_clone(xmpp_conn_t *const conn)
+xmpp_conn_t *xmpp_conn_clone(xmpp_conn_t *conn)
 {
     conn->ref++;
     return conn;
@@ -219,7 +225,7 @@ xmpp_conn_t *xmpp_conn_clone(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-void xmpp_conn_set_keepalive(xmpp_conn_t *const conn, int timeout, int interval)
+void xmpp_conn_set_keepalive(xmpp_conn_t *conn, int timeout, int interval)
 {
     int ret = 0;
 
@@ -245,7 +251,7 @@ void xmpp_conn_set_keepalive(xmpp_conn_t *const conn, int timeout, int interval)
  *
  *  @ingroup Connections
  */
-int xmpp_conn_release(xmpp_conn_t *const conn)
+int xmpp_conn_release(xmpp_conn_t *conn)
 {
     xmpp_ctx_t *ctx;
     xmpp_connlist_t *item, *prev;
@@ -338,6 +344,14 @@ int xmpp_conn_release(xmpp_conn_t *const conn)
             xmpp_free(ctx, conn->pass);
         if (conn->lang)
             xmpp_free(ctx, conn->lang);
+        if (conn->tls_client_cert)
+            xmpp_free(ctx, conn->tls_client_cert);
+        if (conn->tls_client_key)
+            xmpp_free(ctx, conn->tls_client_key);
+        if (conn->tls_cafile)
+            xmpp_free(ctx, conn->tls_cafile);
+        if (conn->tls_capath)
+            xmpp_free(ctx, conn->tls_capath);
         xmpp_free(ctx, conn);
         released = 1;
     }
@@ -353,7 +367,7 @@ int xmpp_conn_release(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-const char *xmpp_conn_get_jid(const xmpp_conn_t *const conn)
+const char *xmpp_conn_get_jid(const xmpp_conn_t *conn)
 {
     return conn->jid;
 }
@@ -371,7 +385,7 @@ const char *xmpp_conn_get_jid(const xmpp_conn_t *const conn)
  *
  * @ingroup Connections
  */
-const char *xmpp_conn_get_bound_jid(const xmpp_conn_t *const conn)
+const char *xmpp_conn_get_bound_jid(const xmpp_conn_t *conn)
 {
     return conn->bound_jid;
 }
@@ -387,13 +401,126 @@ const char *xmpp_conn_get_bound_jid(const xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-void xmpp_conn_set_jid(xmpp_conn_t *const conn, const char *const jid)
+void xmpp_conn_set_jid(xmpp_conn_t *conn, const char *jid)
 {
     if (conn->jid)
         xmpp_free(conn->ctx, conn->jid);
     conn->jid = xmpp_strdup(conn->ctx, jid);
 }
 
+/** Set the Handler function which will be called when the TLS stack can't
+ *  verify the CA of the server we're trying to connect to.
+ *
+ *  @param conn a Strophe connection object
+ *  @param hndl certfail Handler function
+ *
+ *  @ingroup Connections
+ *  @ingroup TLS
+ */
+void xmpp_conn_set_certfail_handler(xmpp_conn_t *const conn,
+                                    xmpp_certfail_handler hndl)
+{
+    conn->certfail_handler = hndl;
+}
+
+/** Set the CAfile
+ *
+ *  @param conn a Strophe connection object
+ *  @param cert path to a certificate file
+ *
+ *  @ingroup Connections
+ *  @ingroup TLS
+ */
+void xmpp_conn_set_cafile(xmpp_conn_t *const conn, const char *path)
+{
+    conn->tls_cafile = xmpp_strdup(conn->ctx, path);
+}
+
+/** Set the CApath
+ *
+ *  @param conn a Strophe connection object
+ *  @param cert path to a folder containing certificates
+ *
+ *  @ingroup Connections
+ *  @ingroup TLS
+ */
+void xmpp_conn_set_capath(xmpp_conn_t *const conn, const char *path)
+{
+    conn->tls_capath = xmpp_strdup(conn->ctx, path);
+}
+
+/** Retrieve the peer certificate
+ *
+ *  The returned Certificate object must be free'd by calling
+ *  \ref xmpp_tlscert_free
+ *
+ *  @param conn a Strophe connection object
+ *
+ *  @return a Strophe Certificate object
+ *
+ *  @ingroup Connections
+ *  @ingroup TLS
+ */
+xmpp_tlscert_t *xmpp_conn_get_peer_cert(xmpp_conn_t *const conn)
+{
+    return tls_peer_cert(conn);
+}
+
+/** Set the Client Certificate and Private Key that will be bound to the
+ *  connection. If any of the both was previously set, it will be discarded.
+ *  This should not be used after a connection is created.  The function will
+ *  make a copy of the strings passed in.
+ *  Currently only non-encrypted Private Keys are supported.
+ *
+ *  @param conn a Strophe connection object
+ *  @param cert path to a certificate file
+ *  @param key path to a private key file
+ *
+ *  @ingroup Connections
+ *  @ingroup TLS
+ */
+void xmpp_conn_set_client_cert(xmpp_conn_t *const conn,
+                               const char *const cert,
+                               const char *const key)
+{
+    xmpp_debug(conn->ctx, "conn", "set client cert %s %s", cert, key);
+    if (conn->tls_client_cert)
+        xmpp_free(conn->ctx, conn->tls_client_cert);
+    conn->tls_client_cert = xmpp_strdup(conn->ctx, cert);
+    if (conn->tls_client_key)
+        xmpp_free(conn->ctx, conn->tls_client_key);
+    conn->tls_client_key = xmpp_strdup(conn->ctx, key);
+}
+
+/** Get the number of xmppAddr entries in the client certificate.
+ *
+ *  @param conn a Strophe connection object
+ *
+ *  @return the number of xmppAddr entries in the client certificate
+ *
+ *  @ingroup Connections
+ *  @ingroup TLS
+ */
+unsigned int xmpp_conn_cert_xmppaddr_num(xmpp_conn_t *const conn)
+{
+    return tls_id_on_xmppaddr_num(conn);
+}
+
+/** Get a specific xmppAddr entry.
+ *
+ *  @param conn a Strophe connection object
+ *  @param n the index of the entry, starting at 0
+ *
+ *  @return a string containing the xmppAddr or NULL if n is out of range
+ *
+ *  @ingroup Connections
+ *  @ingroup TLS
+ */
+char *xmpp_conn_cert_xmppaddr(xmpp_conn_t *const conn, unsigned int n)
+{
+    return tls_id_on_xmppaddr(conn, n);
+}
+
 /** Get the password used for authentication of a connection.
  *
  *  @param conn a Strophe connection object
@@ -402,7 +529,7 @@ void xmpp_conn_set_jid(xmpp_conn_t *const conn, const char *const jid)
  *
  *  @ingroup Connections
  */
-const char *xmpp_conn_get_pass(const xmpp_conn_t *const conn)
+const char *xmpp_conn_get_pass(const xmpp_conn_t *conn)
 {
     return conn->pass;
 }
@@ -416,7 +543,7 @@ const char *xmpp_conn_get_pass(const xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-void xmpp_conn_set_pass(xmpp_conn_t *const conn, const char *const pass)
+void xmpp_conn_set_pass(xmpp_conn_t *conn, const char *pass)
 {
     if (conn->pass)
         xmpp_free(conn->ctx, conn->pass);
@@ -430,7 +557,7 @@ void xmpp_conn_set_pass(xmpp_conn_t *const conn, const char *const pass)
  *
  *  @ingroup Connections
  */
-xmpp_ctx_t *xmpp_conn_get_context(xmpp_conn_t *const conn)
+xmpp_ctx_t *xmpp_conn_get_context(xmpp_conn_t *conn)
 {
     return conn->ctx;
 }
@@ -456,11 +583,11 @@ xmpp_ctx_t *xmpp_conn_get_context(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-int xmpp_connect_client(xmpp_conn_t *const conn,
-                        const char *const altdomain,
+int xmpp_connect_client(xmpp_conn_t *conn,
+                        const char *altdomain,
                         unsigned short altport,
                         xmpp_conn_handler callback,
-                        void *const userdata)
+                        void *userdata)
 {
     resolver_srv_rr_t *srv_rr_list = NULL;
     resolver_srv_rr_t *rr;
@@ -470,6 +597,25 @@ int xmpp_connect_client(xmpp_conn_t *const conn,
     int found = XMPP_DOMAIN_NOT_FOUND;
     int rc;
 
+    if (!conn->jid && conn->tls_client_cert) {
+        if (tls_id_on_xmppaddr_num(conn) != 1) {
+            xmpp_debug(conn->ctx, "xmpp",
+                       "Client certificate contains multiple or no xmppAddr "
+                       "and no JID was given to be used.");
+            return XMPP_EINVOP;
+        }
+        conn->jid = tls_id_on_xmppaddr(conn, 0);
+        if (!conn->jid)
+            return XMPP_EMEM;
+        xmpp_debug(conn->ctx, "xmpp", "Use jid %s from id-on-xmppAddr.",
+                   conn->jid);
+    }
+
+    if (!conn->jid) {
+        xmpp_error(conn->ctx, "xmpp", "JID is not set.");
+        return XMPP_EINVOP;
+    }
+
     domain = xmpp_jid_domain(conn->ctx, conn->jid);
     if (!domain)
         return XMPP_EMEM;
@@ -536,11 +682,11 @@ int xmpp_connect_client(xmpp_conn_t *const conn,
  *
  *  @ingroup Connections
  */
-int xmpp_connect_component(xmpp_conn_t *const conn,
-                           const char *const server,
+int xmpp_connect_component(xmpp_conn_t *conn,
+                           const char *server,
                            unsigned short port,
                            xmpp_conn_handler callback,
-                           void *const userdata)
+                           void *userdata)
 {
     /*  The server domain, jid and password MUST be specified. */
     if (!(server && conn->jid && conn->pass))
@@ -583,18 +729,18 @@ int xmpp_connect_component(xmpp_conn_t *const conn,
  *
  *  @ingroup Connections
  */
-int xmpp_connect_raw(xmpp_conn_t *const conn,
-                     const char *const altdomain,
+int xmpp_connect_raw(xmpp_conn_t *conn,
+                     const char *altdomain,
                      unsigned short altport,
                      xmpp_conn_handler callback,
-                     void *const userdata)
+                     void *userdata)
 {
     conn->is_raw = 1;
     return xmpp_connect_client(conn, altdomain, altport, callback, userdata);
 }
 
 /* Called when tcp connection is established. */
-void conn_established(xmpp_conn_t *const conn)
+void conn_established(xmpp_conn_t *conn)
 {
     if (conn->tls_legacy_ssl && !conn->is_raw) {
         xmpp_debug(conn->ctx, "xmpp", "using legacy SSL connection");
@@ -628,7 +774,7 @@ void conn_established(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-int xmpp_conn_open_stream_default(xmpp_conn_t *const conn)
+int xmpp_conn_open_stream_default(xmpp_conn_t *conn)
 {
     if (!conn->is_raw)
         return XMPP_EINVOP;
@@ -655,24 +801,16 @@ int xmpp_conn_open_stream_default(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-int xmpp_conn_open_stream(xmpp_conn_t *const conn,
+int xmpp_conn_open_stream(xmpp_conn_t *conn,
                           char **attributes,
                           size_t attributes_len)
 {
-    char *tag;
-
     if (!conn->is_raw)
         return XMPP_EINVOP;
 
-    tag = _conn_build_stream_tag(conn, attributes, attributes_len);
-    if (!tag)
-        return XMPP_EMEM;
-
     conn_prepare_reset(conn, auth_handle_open_raw);
-    xmpp_send_raw_string(conn, "<?xml version=\"1.0\"?>%s", tag);
-    xmpp_free(conn->ctx, tag);
 
-    return XMPP_EOK;
+    return _conn_open_stream_with_attributes(conn, attributes, attributes_len);
 }
 
 /** Start synchronous TLS handshake with the server.
@@ -681,7 +819,7 @@ int xmpp_conn_open_stream(xmpp_conn_t *const conn,
  *
  *  @ingroup Connections
  */
-int xmpp_conn_tls_start(xmpp_conn_t *const conn)
+int xmpp_conn_tls_start(xmpp_conn_t *conn)
 {
     return conn_tls_start(conn);
 }
@@ -692,7 +830,7 @@ int xmpp_conn_tls_start(xmpp_conn_t *const conn)
  *
  *  @param conn a Strophe connection object
  */
-void conn_disconnect_clean(xmpp_conn_t *const conn)
+void conn_disconnect_clean(xmpp_conn_t *conn)
 {
     /* remove the timed handler */
     xmpp_timed_handler_delete(conn, _disconnect_cleanup);
@@ -706,7 +844,7 @@ void conn_disconnect_clean(xmpp_conn_t *const conn)
  *
  *  @param conn a Strophe connection object
  */
-void conn_disconnect(xmpp_conn_t *const conn)
+void conn_disconnect(xmpp_conn_t *conn)
 {
     xmpp_debug(conn->ctx, "xmpp", "Closing socket.");
     conn->state = XMPP_STATE_DISCONNECTED;
@@ -724,14 +862,14 @@ void conn_disconnect(xmpp_conn_t *const conn)
 
 /* prepares a parser reset.  this is called from handlers. we can't
  * reset the parser immediately as it is not re-entrant. */
-void conn_prepare_reset(xmpp_conn_t *const conn, xmpp_open_handler handler)
+void conn_prepare_reset(xmpp_conn_t *conn, xmpp_open_handler handler)
 {
     conn->reset_parser = 1;
     conn->open_handler = handler;
 }
 
 /* reset the parser */
-void conn_parser_reset(xmpp_conn_t *const conn)
+void conn_parser_reset(xmpp_conn_t *conn)
 {
     conn->reset_parser = 0;
     parser_reset(conn->parser);
@@ -746,7 +884,7 @@ void conn_parser_reset(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-void xmpp_disconnect(xmpp_conn_t *const conn)
+void xmpp_disconnect(xmpp_conn_t *conn)
 {
     if (conn->state != XMPP_STATE_CONNECTING &&
         conn->state != XMPP_STATE_CONNECTED)
@@ -772,13 +910,16 @@ void xmpp_disconnect(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-void xmpp_send_raw_string(xmpp_conn_t *const conn, const char *const fmt, ...)
+void xmpp_send_raw_string(xmpp_conn_t *conn, const char *fmt, ...)
 {
     va_list ap;
     size_t len;
     char buf[1024]; /* small buffer for common case */
     char *bigbuf;
 
+    if (conn->state != XMPP_STATE_CONNECTED)
+        return;
+
     va_start(ap, fmt);
     len = xmpp_vsnprintf(buf, sizeof(buf), fmt, ap);
     va_end(ap);
@@ -797,14 +938,10 @@ void xmpp_send_raw_string(xmpp_conn_t *const conn, const char *const fmt, ...)
         xmpp_vsnprintf(bigbuf, len, fmt, ap);
         va_end(ap);
 
-        xmpp_debug(conn->ctx, "conn", "SENT: %s", bigbuf);
-
         /* len - 1 so we don't send trailing \0 */
-        xmpp_send_raw(conn, bigbuf, len - 1);
-
-        xmpp_free(conn->ctx, bigbuf);
+        _send_raw(conn, bigbuf, len - 1);
     } else {
-        xmpp_debug(conn->ctx, "conn", "SENT: %s", buf);
+        /* go through xmpp_send_raw() which does the strdup() for us */
         xmpp_send_raw(conn, buf, len);
     }
 }
@@ -821,41 +958,20 @@ void xmpp_send_raw_string(xmpp_conn_t *const conn, const char *const fmt, ...)
  *
  *  @ingroup Connections
  */
-void xmpp_send_raw(xmpp_conn_t *const conn,
-                   const char *const data,
-                   const size_t len)
+void xmpp_send_raw(xmpp_conn_t *conn, const char *data, size_t len)
 {
-    xmpp_send_queue_t *item;
+    char *d;
 
     if (conn->state != XMPP_STATE_CONNECTED)
         return;
 
-    /* create send queue item for queue */
-    item = xmpp_alloc(conn->ctx, sizeof(xmpp_send_queue_t));
-    if (!item)
-        return;
-
-    item->data = xmpp_alloc(conn->ctx, len);
-    if (!item->data) {
-        xmpp_free(conn->ctx, item);
+    d = xmpp_strndup(conn->ctx, data, len);
+    if (!d) {
+        xmpp_error(conn->ctx, "conn", "Failed to strndup");
         return;
     }
-    memcpy(item->data, data, len);
-    item->len = len;
-    item->next = NULL;
-    item->written = 0;
 
-    /* add item to the send queue */
-    if (!conn->send_queue_tail) {
-        /* first item, set head and tail */
-        conn->send_queue_head = item;
-        conn->send_queue_tail = item;
-    } else {
-        /* add to the tail */
-        conn->send_queue_tail->next = item;
-        conn->send_queue_tail = item;
-    }
-    conn->send_queue_len++;
+    _send_raw(conn, d, len);
 }
 
 /** Send an XML stanza to the XMPP server.
@@ -867,18 +983,20 @@ void xmpp_send_raw(xmpp_conn_t *const conn,
  *
  *  @ingroup Connections
  */
-void xmpp_send(xmpp_conn_t *const conn, xmpp_stanza_t *const stanza)
+void xmpp_send(xmpp_conn_t *conn, xmpp_stanza_t *stanza)
 {
-    char *buf;
+    char *buf = NULL;
     size_t len;
 
-    if (conn->state == XMPP_STATE_CONNECTED) {
-        if (xmpp_stanza_to_text(stanza, &buf, &len) == 0) {
-            xmpp_send_raw(conn, buf, len);
-            xmpp_debug(conn->ctx, "conn", "SENT: %s", buf);
-            xmpp_free(conn->ctx, buf);
-        }
+    if (conn->state != XMPP_STATE_CONNECTED)
+        return;
+
+    if (xmpp_stanza_to_text(stanza, &buf, &len) != 0) {
+        xmpp_error(conn->ctx, "conn", "Failed to stanza_to_text");
+        return;
     }
+
+    _send_raw(conn, buf, len);
 }
 
 /** Send the opening &lt;stream:stream&gt; tag to the server.
@@ -887,22 +1005,36 @@ void xmpp_send(xmpp_conn_t *const conn, xmpp_stanza_t *const stanza)
  *
  *  @param conn a Strophe connection object
  */
-void conn_open_stream(xmpp_conn_t *const conn)
+void conn_open_stream(xmpp_conn_t *conn)
 {
-    xmpp_send_raw_string(conn,
-                         "<?xml version=\"1.0\"?>"
-                         "<stream:stream to=\"%s\" "
-                         "xml:lang=\"%s\" "
-                         "version=\"1.0\" "
-                         "xmlns=\"%s\" "
-                         "xmlns:stream=\"%s\">",
-                         conn->domain, conn->lang,
-                         conn->type == XMPP_CLIENT ? XMPP_NS_CLIENT
-                                                   : XMPP_NS_COMPONENT,
-                         XMPP_NS_STREAMS);
+    size_t attributes_len;
+    int rc;
+    char *from = NULL;
+    char *ns = conn->type == XMPP_CLIENT ? XMPP_NS_CLIENT : XMPP_NS_COMPONENT;
+    char *attributes[12] = {
+        "to",           conn->domain,    "xml:lang", conn->lang,
+        "version",      "1.0",           "xmlns",    ns,
+        "xmlns:stream", XMPP_NS_STREAMS, "from",     NULL};
+
+    attributes_len = ARRAY_SIZE(attributes);
+    if (conn->tls && conn->jid && strchr(conn->jid, '@') != NULL)
+        from = xmpp_jid_bare(conn->ctx, conn->jid);
+
+    if (from)
+        attributes[attributes_len - 1] = from;
+    else
+        attributes_len -= 2;
+
+    rc = _conn_open_stream_with_attributes(conn, attributes, attributes_len);
+    if (rc != XMPP_EOK) {
+        xmpp_error(conn->ctx, "conn", "Cannot build stream tag: memory error");
+        conn_disconnect(conn);
+    }
+    if (from)
+        xmpp_free(conn->ctx, from);
 }
 
-int conn_tls_start(xmpp_conn_t *const conn)
+int conn_tls_start(xmpp_conn_t *conn)
 {
     int rc;
 
@@ -942,7 +1074,7 @@ int conn_tls_start(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-long xmpp_conn_get_flags(const xmpp_conn_t *const conn)
+long xmpp_conn_get_flags(const xmpp_conn_t *conn)
 {
     long flags;
 
@@ -977,7 +1109,7 @@ long xmpp_conn_get_flags(const xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-int xmpp_conn_set_flags(xmpp_conn_t *const conn, long flags)
+int xmpp_conn_set_flags(xmpp_conn_t *conn, long flags)
 {
     if (conn->state != XMPP_STATE_DISCONNECTED) {
         xmpp_error(conn->ctx, "conn",
@@ -1012,7 +1144,7 @@ int xmpp_conn_set_flags(xmpp_conn_t *const conn, long flags)
  *
  *  @ingroup Connections
  */
-void xmpp_conn_disable_tls(xmpp_conn_t *const conn)
+void xmpp_conn_disable_tls(xmpp_conn_t *conn)
 {
     long flags = xmpp_conn_get_flags(conn);
 
@@ -1026,7 +1158,7 @@ void xmpp_conn_disable_tls(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-int xmpp_conn_is_secured(xmpp_conn_t *const conn)
+int xmpp_conn_is_secured(xmpp_conn_t *conn)
 {
     return conn->secured && !conn->tls_failed && conn->tls != NULL;
 }
@@ -1036,7 +1168,7 @@ int xmpp_conn_is_secured(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-int xmpp_conn_is_connecting(xmpp_conn_t *const conn)
+int xmpp_conn_is_connecting(xmpp_conn_t *conn)
 {
     return conn->state == XMPP_STATE_CONNECTING;
 }
@@ -1046,7 +1178,7 @@ int xmpp_conn_is_connecting(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-int xmpp_conn_is_connected(xmpp_conn_t *const conn)
+int xmpp_conn_is_connected(xmpp_conn_t *conn)
 {
     return conn->state == XMPP_STATE_CONNECTED;
 }
@@ -1056,13 +1188,13 @@ int xmpp_conn_is_connected(xmpp_conn_t *const conn)
  *
  *  @ingroup Connections
  */
-int xmpp_conn_is_disconnected(xmpp_conn_t *const conn)
+int xmpp_conn_is_disconnected(xmpp_conn_t *conn)
 {
     return conn->state == XMPP_STATE_DISCONNECTED;
 }
 
 /* timed handler for cleanup if normal disconnect procedure takes too long */
-static int _disconnect_cleanup(xmpp_conn_t *const conn, void *const userdata)
+static int _disconnect_cleanup(xmpp_conn_t *conn, void *userdata)
 {
     UNUSED(userdata);
 
@@ -1073,7 +1205,7 @@ static int _disconnect_cleanup(xmpp_conn_t *const conn, void *const userdata)
     return 0;
 }
 
-static char *_conn_build_stream_tag(xmpp_conn_t *const conn,
+static char *_conn_build_stream_tag(xmpp_conn_t *conn,
                                     char **attributes,
                                     size_t attributes_len)
 {
@@ -1118,6 +1250,22 @@ static char *_conn_build_stream_tag(xmpp_conn_t *const conn,
     return tag;
 }
 
+static int _conn_open_stream_with_attributes(xmpp_conn_t *conn,
+                                             char **attributes,
+                                             size_t attributes_len)
+{
+    char *tag;
+
+    tag = _conn_build_stream_tag(conn, attributes, attributes_len);
+    if (!tag)
+        return XMPP_EMEM;
+
+    xmpp_send_raw_string(conn, "<?xml version=\"1.0\"?>%s", tag);
+    xmpp_free(conn->ctx, tag);
+
+    return XMPP_EOK;
+}
+
 static void _conn_attributes_new(xmpp_conn_t *conn,
                                  char **attrs,
                                  char ***attributes,
@@ -1190,7 +1338,7 @@ static char *_get_stream_attribute(char **attrs, char *name)
     return NULL;
 }
 
-static void _handle_stream_start(char *name, char **attrs, void *const userdata)
+static void _handle_stream_start(char *name, char **attrs, void *userdata)
 {
     xmpp_conn_t *conn = (xmpp_conn_t *)userdata;
     char *id;
@@ -1226,7 +1374,7 @@ static void _handle_stream_start(char *name, char **attrs, void *const userdata)
     }
 }
 
-static void _handle_stream_end(char *name, void *const userdata)
+static void _handle_stream_end(char *name, void *userdata)
 {
     xmpp_conn_t *conn = (xmpp_conn_t *)userdata;
 
@@ -1237,7 +1385,7 @@ static void _handle_stream_end(char *name, void *const userdata)
     conn_disconnect_clean(conn);
 }
 
-static void _handle_stream_stanza(xmpp_stanza_t *stanza, void *const userdata)
+static void _handle_stream_stanza(xmpp_stanza_t *stanza, void *userdata)
 {
     xmpp_conn_t *conn = (xmpp_conn_t *)userdata;
     char *buf;
@@ -1251,7 +1399,7 @@ static void _handle_stream_stanza(xmpp_stanza_t *stanza, void *const userdata)
     handler_fire_stanza(conn, stanza);
 }
 
-static unsigned short _conn_default_port(xmpp_conn_t *const conn,
+static unsigned short _conn_default_port(xmpp_conn_t *conn,
                                          xmpp_conn_type_t type)
 {
     switch (type) {
@@ -1265,7 +1413,7 @@ static unsigned short _conn_default_port(xmpp_conn_t *const conn,
     };
 }
 
-static void _conn_reset(xmpp_conn_t *const conn)
+static void _conn_reset(xmpp_conn_t *conn)
 {
     xmpp_ctx_t *ctx = conn->ctx;
     xmpp_send_queue_t *sq, *tsq;
@@ -1316,13 +1464,13 @@ static void _conn_reset(xmpp_conn_t *const conn)
     handler_system_delete_all(conn);
 }
 
-static int _conn_connect(xmpp_conn_t *const conn,
-                         const char *const domain,
-                         const char *const host,
+static int _conn_connect(xmpp_conn_t *conn,
+                         const char *domain,
+                         const char *host,
                          unsigned short port,
                          xmpp_conn_type_t type,
                          xmpp_conn_handler callback,
-                         void *const userdata)
+                         void *userdata)
 {
     xmpp_open_handler open_handler;
 
@@ -1352,10 +1500,9 @@ static int _conn_connect(xmpp_conn_t *const conn,
     conn->conn_handler = callback;
     conn->userdata = userdata;
 
-    open_handler = conn->is_raw
-                       ? auth_handle_open_stub
-                       : type == XMPP_CLIENT ? auth_handle_open
-                                             : auth_handle_component_open;
+    open_handler = conn->is_raw          ? auth_handle_open_stub
+                   : type == XMPP_CLIENT ? auth_handle_open
+                                         : auth_handle_component_open;
     conn_prepare_reset(conn, open_handler);
 
     /* FIXME: it could happen that the connect returns immediately as
@@ -1369,3 +1516,36 @@ static int _conn_connect(xmpp_conn_t *const conn,
 
     return 0;
 }
+
+static int _send_raw(xmpp_conn_t *conn, char *data, size_t len)
+{
+    xmpp_send_queue_t *item;
+
+    /* create send queue item for queue */
+    item = xmpp_alloc(conn->ctx, sizeof(xmpp_send_queue_t));
+    if (!item) {
+        xmpp_error(conn->ctx, "conn", "DROPPED: %s", data);
+        xmpp_free(conn->ctx, data);
+        return XMPP_EMEM;
+    }
+
+    item->data = data;
+    item->len = len;
+    item->next = NULL;
+    item->written = 0;
+
+    /* add item to the send queue */
+    if (!conn->send_queue_tail) {
+        /* first item, set head and tail */
+        conn->send_queue_head = item;
+        conn->send_queue_tail = item;
+    } else {
+        /* add to the tail */
+        conn->send_queue_tail->next = item;
+        conn->send_queue_tail = item;
+    }
+    conn->send_queue_len++;
+    xmpp_debug_verbose(2, conn->ctx, "conn", "QUEUED: %s", data);
+    xmpp_debug_verbose(1, conn->ctx, "conn", "Added queue element: %p", item);
+    return XMPP_EOK;
+}
