@@ -73,7 +73,7 @@
  *
  *  @ingroup EventLoop
  */
-void xmpp_run_once(xmpp_ctx_t *ctx, const unsigned long timeout)
+void xmpp_run_once(xmpp_ctx_t *ctx, unsigned long timeout)
 {
     xmpp_connlist_t *connitem;
     xmpp_conn_t *conn;
@@ -110,6 +110,7 @@ void xmpp_run_once(xmpp_ctx_t *ctx, const unsigned long timeout)
                 xmpp_debug(ctx, "xmpp", "Send error occurred, disconnecting.");
                 conn->error = ECONNABORTED;
                 conn_disconnect(conn);
+                goto NEXT_ITEM;
             }
         }
 
@@ -133,6 +134,9 @@ void xmpp_run_once(xmpp_ctx_t *ctx, const unsigned long timeout)
                 break; /* partial write or an error */
 
             /* all data for this queue item written, delete and move on */
+            xmpp_debug(conn->ctx, "conn", "SENT: %s", sq->data);
+            xmpp_debug_verbose(1, ctx, "xmpp",
+                               "Finished writing queue element: %p.", sq);
             xmpp_free(ctx, sq->data);
             tsq = sq;
             sq = sq->next;
@@ -154,7 +158,7 @@ void xmpp_run_once(xmpp_ctx_t *ctx, const unsigned long timeout)
             conn->error = ECONNABORTED;
             conn_disconnect(conn);
         }
-
+NEXT_ITEM:
         connitem = connitem->next;
     }
 
