@@ -1,7 +1,7 @@
 /* check_parser.h
 ** strophe XMPP client library -- parser tests
 **
-** Copyright (C) 2005-2009 Collecta, Inc. 
+** Copyright (C) 2005-2009 Collecta, Inc.
 **
 **  This software is provided AS-IS with no warranty, either express or
 **  implied.
@@ -18,14 +18,15 @@
 
 #include "test.h"
 
-#define fail_unless(expr) do {                  \
-    int result = (expr);                        \
-    if (!result) {                              \
-        printf("%s:%d: Assertion failed: %s\n", \
-               __FILE__, __LINE__, #expr);      \
-        exit(1);                                \
-    }                                           \
-} while (0)
+#define fail_unless(expr)                                               \
+    do {                                                                \
+        int result = (expr);                                            \
+        if (!result) {                                                  \
+            printf("%s:%d: Assertion failed: %s\n", __FILE__, __LINE__, \
+                   #expr);                                              \
+            exit(1);                                                    \
+        }                                                               \
+    } while (0)
 
 static void create_destroy(void)
 {
@@ -42,6 +43,9 @@ static void create_destroy(void)
 int cbtest_got_start = 0;
 void cbtest_handle_start(char *name, char **attrs, void *userdata)
 {
+    (void)attrs;
+    (void)userdata;
+
     if (strcmp(name, "stream") == 0)
         cbtest_got_start = 1;
 }
@@ -49,6 +53,8 @@ void cbtest_handle_start(char *name, char **attrs, void *userdata)
 int cbtest_got_end = 0;
 void cbtest_handle_end(char *name, void *userdata)
 {
+    (void)userdata;
+
     if (strcmp(name, "stream") == 0)
         cbtest_got_end = 1;
 }
@@ -56,6 +62,8 @@ void cbtest_handle_end(char *name, void *userdata)
 int cbtest_got_stanza = 0;
 void cbtest_handle_stanza(xmpp_stanza_t *stanza, void *userdata)
 {
+    (void)userdata;
+
     if (strcmp(xmpp_stanza_get_name(stanza), "message") == 0)
         cbtest_got_stanza = 1;
 }
@@ -67,9 +75,7 @@ static void callbacks(void)
     int ret;
 
     ctx = xmpp_ctx_new(NULL, NULL);
-    parser = parser_new(ctx, 
-                        cbtest_handle_start, 
-                        cbtest_handle_end,
+    parser = parser_new(ctx, cbtest_handle_start, cbtest_handle_end,
                         cbtest_handle_stanza, NULL);
 
     ret = parser_feed(parser, "<stream>", 8);
