@@ -21,19 +21,20 @@
 #include "rand.c"
 
 /* stubs to build test without whole libstrophe */
-void *xmpp_alloc(const xmpp_ctx_t *const ctx, const size_t size)
+void *xmpp_alloc(const xmpp_ctx_t *ctx, size_t size)
 {
     (void)ctx;
     (void)size;
     return NULL;
 }
 
-void xmpp_free(const xmpp_ctx_t *const ctx, void *p)
+void xmpp_free(const xmpp_ctx_t *ctx, void *p)
 {
     (void)ctx;
     (void)p;
 }
 
+#ifndef HAVE_SNPRINTF
 int xmpp_snprintf(char *str, size_t count, const char *fmt, ...)
 {
     (void)str;
@@ -41,6 +42,7 @@ int xmpp_snprintf(char *str, size_t count, const char *fmt, ...)
     (void)fmt;
     return 0;
 }
+#endif /* HAVE_SNPRINTF */
 
 uint64_t time_stamp(void)
 {
