@@ -23,11 +23,11 @@
 /* strtok_s() has appeared in visual studio 2005.
    Use own implementation for older versions. */
 #ifdef _MSC_VER
-# if (_MSC_VER >= 1400)
-# define strtok_r strtok_s
-# else
-# define strtok_r xmpp_strtok_r
-# endif
+#if (_MSC_VER >= 1400)
+#define strtok_r strtok_s
+#else
+#define strtok_r xmpp_strtok_r
+#endif
 #endif /* _MSC_VER */
 
 static int test_strtok_r(void)
@@ -75,8 +75,8 @@ static int test_strdup_one(xmpp_ctx_t *ctx, const char *s)
 
     if (!s1 || !s2 || strcmp(s1, s2) != 0) {
         rc = -1;
-        printf("strdup: '%s', xmpp_strdup: '%s'\n",
-               s1 ? s1 : "<NULL>", s2 ? s2 : "<NULL>");
+        printf("strdup: '%s', xmpp_strdup: '%s'\n", s1 ? s1 : "<NULL>",
+               s2 ? s2 : "<NULL>");
     }
 
     free(s1);
@@ -92,7 +92,7 @@ static int test_strdup(void)
     size_t i;
     int rc = 0;
 
-    static const char *tests[] = { "", "\0", "test", "s p a c e", "\n\r" };
+    static const char *tests[] = {"", "\0", "test", "s p a c e", "\n\r"};
 
     ctx = xmpp_ctx_new(NULL, NULL);
     assert(ctx != NULL);
